@@ -5,6 +5,7 @@ import sys
 import types
 import typing as t
 import weakref
+from collections import ChainMap
 from functools import singledispatch
 from importlib import import_module
 
@@ -132,7 +133,7 @@ class _LazyEvaluator(ast.NodeTransformer):
                            None
                            if node.annotation is None
                            else ast.Str(conversion.to_str(node.annotation)),
-                           node.type_comment)
+                           getattr(node, 'type_comment', None))
 
 
 _AstNode = t.TypeVar('_AstNode',
@@ -146,7 +147,7 @@ def _(ast_node: t.Union[ast.AsyncFunctionDef, ast.FunctionDef],
       module_path: catalog.Path,
       parent_path: catalog.Path,
       parent_namespace: namespacing.Namespace) -> t.Any:
-    namespace = parent_namespace.copy()
+    namespace = dict(parent_namespace)
     try:
         source_path = sources.from_module_path(module_path)
     except sources.NotFound:
@@ -193,9 +194,23 @@ def _(ast_node: ast.List,
 def _(ast_node: ast.Assign,
       module_path: catalog.Path,
       parent_path: catalog.Path,
-      parent_namespace: namespacing.Namespace) -> t.Any:
-    return evaluate_expression_node(ast_node.value, module_path, parent_path,
-                                    parent_namespace)
+      parent_namespace: namespacing.Namespace,
+      *,
+      namespace: namespacing.Namespace = globals()) -> t.Any:
+    names = conversion.to_names(ast_node)
+    for name in names:
+        try:
+            result = namespace[name]
+        except KeyError:
+            continue
+        else:
+            break
+    else:
+        result = evaluate_expression_node(ast_node.value, module_path,
+                                          parent_path, parent_namespace)
+    for name in names:
+        namespace[name] = result
+    return result
 
 
 @evaluate_statement_node.register(ast.AnnAssign)
@@ -235,8 +250,8 @@ def _(ast_node: ast.Name,
             module_path, parent_path, (object_name,), stubs.definitions,
             stubs.references, stubs.submodules, stubs.superclasses
     )
-    return _evaluate_qualified_path(module_path, object_path,
-                                    parent_namespace)
+    return evaluate_qualified_path(module_path, object_path,
+                                   parent_namespace)
 
 
 @evaluate_expression_node.register(ast.Attribute)
@@ -255,11 +270,11 @@ def _(ast_node: ast.Attribute,
                 stubs.definitions, stubs.references, stubs.submodules,
                 stubs.superclasses
         )
-        return _evaluate_qualified_path(module_path, object_path,
-                                        parent_namespace)
+        return evaluate_qualified_path(module_path, object_path,
+                                       parent_namespace)
 
 
-def _evaluate_qualified_path(
+def evaluate_qualified_path(
         module_path: catalog.Path,
         object_path: catalog.Path,
         parent_namespace: namespacing.Namespace,
@@ -282,11 +297,11 @@ def _evaluate_qualified_path(
                 modules_cache[module_path] = module = types.ModuleType(
                         module_name
                 )
-                module_raw_namespace = module.__dict__
+                namespace = module.__dict__
                 for name in stubs.definitions[module_path].keys():
-                    module_raw_namespace[name] = parent_namespace[name] = (
-                        _evaluate_qualified_path(module_path, (name,),
-                                                 parent_namespace)
+                    namespace[name] = parent_namespace[name] = (
+                        evaluate_qualified_path(module_path, (name,),
+                                                parent_namespace)
                     )
                 return module
     else:
@@ -310,9 +325,8 @@ def _evaluate_qualified_path(
             scope = stubs.definitions[module_path]
             for part in object_path:
                 scope = scope[part]
-            class_raw_namespace: namespacing.Namespace = {
-                '__module__': module_name
-            }
+            class_namespace: namespacing.Namespace = {'__module__': __name__}
+            module_namespace: namespacing.Namespace = {'__name__': __name__}
             annotations_nodes = {}
             for name in scope.keys():
                 raw_ast_nodes = module_raw_ast_nodes[(*object_path, name)]
@@ -344,23 +358,23 @@ def _evaluate_qualified_path(
                             )
                         )
                         if dependency_module_path != builtins_module_path:
-                            class_raw_namespace[dependency_name] = (
-                                _evaluate_qualified_path(
+                            module_namespace[dependency_name] = (
+                                evaluate_qualified_path(
                                         dependency_module_path,
                                         dependency_object_path,
-                                        class_raw_namespace
+                                        module_namespace
                                 )
                             )
                     value = evaluate_statement_node(
                             definition_node, module_path, object_path,
-                            class_raw_namespace
+                            ChainMap(class_namespace, module_namespace)
                     )
                 if definitions_nodes:
-                    class_raw_namespace[name] = value
+                    class_namespace[name] = value
             bases = tuple(
-                    _evaluate_qualified_path(superclass_module_path,
-                                             superclass_object_path,
-                                             class_raw_namespace)
+                    evaluate_qualified_path(superclass_module_path,
+                                            superclass_object_path,
+                                            module_namespace)
                     for (
                         superclass_module_path, superclass_object_path
                     ) in stubs.superclasses.get(module_path, {}).get(
@@ -368,13 +382,13 @@ def _evaluate_qualified_path(
                     )
             )
             if annotations_nodes:
-                class_raw_namespace['__annotations__'] = {}
+                class_namespace['__annotations__'] = {}
             result = parent_namespace[object_path[-1]] = objects_cache[
                 (module_path, object_path)
             ] = types.new_class(
-                    object_path[-1], bases,
-                    exec_body=lambda raw_namespace: raw_namespace.update(
-                            class_raw_namespace
+                    catalog.path_to_string(object_path), bases,
+                    exec_body=lambda namespace: namespace.update(
+                            class_namespace
                     )
             )
             if annotations_nodes:
@@ -384,6 +398,7 @@ def _evaluate_qualified_path(
                                                    parent_namespace)
                     for name, annotation_node in annotations_nodes.items()
                 })
+            globals()[catalog.path_to_string(object_path)] = result
             return result
         else:
             for raw_ast_node in raw_ast_nodes:
